/*-
 * Copyright (C) @BABOLO  2009 May 08 http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2009 May 08 http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: ipfw_syntax.c,v 1.12 2010/10/22 02:20:49 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <stdio.h>
#include <mife.h>
#include <multilar.h>
#include "bsdfig.h"

#if __FreeBSD_version < 500000
# include <netinet/ip_fw2.h>
# include <net/route.h>
# include <sys/mbuf.h>
#else
# include <netinet/ip_fw.h>
#endif

#define l1    ( 1 << BSDFIG_lSHFT)
#define l2    ( 2 << BSDFIG_lSHFT)
#define l3    ( 3 << BSDFIG_lSHFT)
#define l5    ( 5 << BSDFIG_lSHFT)
#define l6    ( 6 << BSDFIG_lSHFT)
#define l7    ( 7 << BSDFIG_lSHFT)
#define l8    ( 8 << BSDFIG_lSHFT)
#define l9    ( 9 << BSDFIG_lSHFT)
#define L1    ( 1 << BSDFIG_LSHFT)
#define L2    ( 2 << BSDFIG_LSHFT)
#define L3    ( 3 << BSDFIG_LSHFT)
#define L5    ( 5 << BSDFIG_LSHFT)
#define L6    ( 6 << BSDFIG_LSHFT)
#define L7    ( 7 << BSDFIG_LSHFT)
#define L8    ( 8 << BSDFIG_LSHFT)
#define Lp    (31 << BSDFIG_LSHFT)
#define Lm    (63 << BSDFIG_LSHFT)
#define LM    ((1 << BSDFIG_MSHFT) | Lm) /*    */
#define LI    ((2 << BSDFIG_MSHFT) | (57 << BSDFIG_LSHFT)) /*   1 + 8n */

#define As    BSDFIG_ROLAs
#define Ac    BSDFIG_ROLAc
#define Ak    BSDFIG_ROLAk
#define Al    BSDFIG_ROLAl
#define Aq    BSDFIG_ROLAq
#define Ap    BSDFIG_ROLAp
#define At    BSDFIG_ROLAt
#define Aa    BSDFIG_ROLAa
#define An    BSDFIG_ROLAn
#define Amask BSDFIG_ROLAM
#define Nx    BSDFIG_ROLNx

#define Hn BSDFIG_ARGTY(BSDFIG_A1NUM)
#define HN BSDFIG_ARGTY(BSDFIG_A1BIT)
#define Hh BSDFIG_ARGTY(BSDFIG_A163K)
#define H8 BSDFIG_ARGTY(BSDFIG_A1255)
#define H9 BSDFIG_ARGTY(BSDFIG_A1256)
#define Ht BSDFIG_ARGTY(BSDFIG_A1127)
#define Hr BSDFIG_ARGTY(BSDFIG_A1FIB)
#define H2 BSDFIG_ARGTY(BSDFIG_A1_3)
#define Hb BSDFIG_ARGTY(BSDFIG_A1_BB)
#define Vv BSDFIG_ARGTY(BSDFIG_PAVAR)
#define Wh BSDFIG_ARGTY(BSDFIG_WFULL)
#define Wf BSDFIG_ARGTY(BSDFIG_FLOAT)
#define Wi BSDFIG_ARGTY(BSDFIG_WIP4N)
#define Wj BSDFIG_ARGTY(BSDFIG_WIP4H)
#define Wa BSDFIG_ARGTY(BSDFIG_IFA4)
#define W6 BSDFIG_ARGTY(BSDFIG_WIP6N)
#define We BSDFIG_ARGTY(BSDFIG_ETHER)
#define Ws BSDFIG_ARGTY(BSDFIG_SOCK)
#define Wl BSDFIG_ARGTY(BSDFIG_LIMIT)
#define Vp BSDFIG_ARGTY(BSDFIG_PAIRS)
#define Vh BSDFIG_ARGTY(BSDFIG_WORDS)
#define Vt BSDFIG_ARGTY(BSDFIG_TEXT0)
#define VT BSDFIG_ARGTY(BSDFIG_TEXT1)
#define Vb BSDFIG_ARGTY(BSDFIG_BITS0)
#define VB BSDFIG_ARGTY(BSDFIG_BITS1)
#define V4 BSDFIG_ARGTY(BSDFIG_IP4MS)
#define V6 BSDFIG_ARGTY(BSDFIG_IP6MS)
#define Er BSDFIG_ARGTY(BSDFIG_ERROR)

static const bsdfig_syllable Wone[] = {0xFFFF0000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
static const bsdfig_syllable Wtre[] = {0, 0, 0xFFFFFFFF};
static const bsdfig_syllable What[] = {0xFFFC0000};

#if __FreeBSD_version >= 500000
const bsdfig_syntax __bsdfig_s =
#else
const bsdfig_syntax bsdfig_s =
#endif
{ 0001020304050607LLU
, "bsdfig\000S"
, O_LAST_OPCODE
, { {         Lm|l1|       Vt, Wone, "NOP"          , O_NOP          } /* 00 */
  , {      Ac|L2|l2|       Wi, Wone, "IP_SRC"       , O_IP_SRC       }
  , {      Ac|LM|l3|       V4, Wone, "IP_SRC_MASK"  , O_IP_SRC_MASK  }
  , {      Ac|L1|l1|        0, Wone, "IP_SRC_ME"    , O_IP_SRC_ME    }
  , {      Ac|Lm|l3| HN|Wj|VB, NULL, "IP_SRC_SET"   , O_IP_SRC_SET   } /* 04 */
  , {      Ac|L2|l2|       Wi, Wone, "IP_DST"       , O_IP_DST       }
  , {      Ac|LM|l3|       V4, Wone, "IP_DST_MASK"  , O_IP_DST_MASK  }
  , {      Ac|L1|l1|        0, Wone, "IP_DST_ME"    , O_IP_DST_ME    }
  , {      Ac|Lm|l3| HN|Wj|VB, NULL, "IP_DST_SET"   , O_IP_DST_SET   } /* 08 */
  , {      Ac|Lp|l2|       Vp, Wone, "IP_SRCPORT"   , O_IP_SRCPORT   }
  , {      Ac|Lp|l2|       Vp, Wone, "IP_DSTPORT"   , O_IP_DSTPORT   }
  , {      Ac|L1|l1|       H8, Wone, "PROTO"        , O_PROTO        }
  , {      Ac|L7|l7|       We, Wone, "MACADDR2"     , O_MACADDR2     } /* 0C */
  , {      Ac|Lp|l2|       Vp, NULL, "MAC_TYPE"     , O_MAC_TYPE     }
  , {      Ac|L1|l1|        0, Wone, "LAYER2"       , O_LAYER2       }
  , {      Ac|L1|l1|        0, Wone, "IN"           , O_IN           }
  , {      Ac|L1|l1|        0, Wone, "FRAG"         , O_FRAG         } /* 10 */
  , {      Ac|L6|l6|    Wa|VT, Wone, "RECV"         , O_RECV         }
  , {      Ac|L6|l6|    Wa|VT, Wone, "XMIT"         , O_XMIT         }
  , {      Ac|L6|l6|    Wa|VT, Wone, "VIA"          , O_VIA          }
  , {      Ac|L1|l1| Hb      , NULL, "IPOPT"        , O_IPOPT        } /* 14 */
  , {      Ac|Lm|l1|    Vv   , NULL, "IPLEN"        , O_IPLEN        }
  , {      Ac|Lm|l1|    Vv   , NULL, "IPID"         , O_IPID         }
  , {      Ac|L1|l1| Hb      , NULL, "IPTOS"        , O_IPTOS        }
  , {      Ac|L1|l1| H8      , NULL, "IPPRECEDENCE" , O_IPPRECEDENCE } /* 18 */
  , {      Ac|Lm|l1|    Vv   , NULL, "IPTTL"        , O_IPTTL        }
  , {      Ac|L1|l1| H8      , NULL, "IPVER"        , O_IPVER        }
  , {      Ac|L2|l2|       Wh, Wtre, "UID"          , O_UID          }
  , {      Ac|L2|l2|       Wh, Wtre, "GID"          , O_GID          } /* 1C */
  , {      Ac|L1|l1|        0, Wone, "ESTAB"        , O_ESTAB        }
  , {      Ac|L1|l1| Hb      , NULL, "TCPFLAGS"     , O_TCPFLAGS     }
  , {      Ac|L1|l1| Hh      , NULL, "TCPWIN"       , O_TCPWIN       }
  , {      Ac|L2|l2|       Wh, Wone, "TCPSEQ"       , O_TCPSEQ       } /* 20 */
  , {      Ac|L2|l2|       Wh, Wone, "TCPACK"       , O_TCPACK       }
  , {      Ac|L2|l2|       Vb, Wtre, "ICMPTYPE"     , O_ICMPTYPE     }
  , {      Ac|L1|l1| Hb      , NULL, "TCPOPTS"      , O_TCPOPTS      }
  , {      Ac|L1|l1|        0, Wone, "VERREVPATH"   , O_VERREVPATH   } /* 24 */
#if __FreeBSD_version >= 500000
  , {      Ac|L1|l1|        0, Wone, "VERSRCREACH"  , O_VERSRCREACH  }
#endif
  , {   Nx|As|L1|l1|        0, Wone, "PROBE_STATE"  , O_PROBE_STATE  }
  , {   Nx|Ak|L1|l1|        0, Wone, "KEEP_STATE"   , O_KEEP_STATE   }
  , {   Nx|Ak|L2|l2|       Wl, Wone, "LIMIT"        , O_LIMIT        } /* 28 */
  , {Er      |Lm|l1| Hh   |Vh, NULL, "LIMIT_PARENT" , O_LIMIT_PARENT }
  , {   Nx   |L3|l3|       Wh, Wtre, "LOG"          , O_LOG          }
  , {   Nx|Ap|L2|l2|       Wf, Wone, "PROB"         , O_PROB         }
  , {   Nx|Aa|L1|l1|        0, Wone, "CHECK_STATE"  , O_CHECK_STATE  } /* 2C */
  , {   Nx|Aa|L1|l1|        0, Wone, "ACCEPT"       , O_ACCEPT       }
  , {   Nx|Aa|L1|l1|        0, Wone, "DENY"         , O_DENY         }
  , {   Nx|Aa|L1|l1| Hh      , NULL, "REJECT"       , O_REJECT       }
  , {   Nx|Aa|L1|l1|        0, Wone, "COUNT"        , O_COUNT        } /* 30 */
  , {   Nx|Aa|L1|l1| Hh      , NULL, "SKIPTO"       , O_SKIPTO       }
#if __FreeBSD_version >= 500000
  , {   Nx|Aa|L1|l1| Hh      , NULL, "PIPE"         , O_PIPE         }
  , {   Nx|Aa|L1|l1| Hh      , NULL, "QUEUE"        , O_QUEUE        }
#else
  , {   Nx|Aa|L2|l2| Hh      , NULL, "PIPE"         , O_PIPE         }
  , {   Nx|Aa|L2|l2| Hh      , NULL, "QUEUE"        , O_QUEUE        }
#endif
  , {   Nx|Aa|L1|l1| Hh      , NULL, "DIVERT"       , O_DIVERT       } /* 34 */
  , {   Nx|Aa|L1|l1| Hh      , NULL, "TEE"          , O_TEE          }
  , {   Nx|Aa|L5|l5|       Ws, Wone, "FORWARD_IP"   , O_FORWARD_IP   }
  , {Er|Nx|Aa|L1|l1| Hh   |Vh, NULL, "FORWARD_MAC"  , O_FORWARD_MAC  }
#if __FreeBSD_version >= 700000
  , {   Nx|Aa|L2|l2| Hh      , NULL, "NAT"          , O_NAT          } /* 38 */
#endif
#if __FreeBSD_version > 800099
  , {      Ac|L1|l1|        0, Wone, "REASS"        , O_REASS        }
#endif
  , {      Ac|L1|l1|        0, Wone, "IPSEC"        , O_IPSEC        }
#if __FreeBSD_version >= 800500
  , {      Ac|L3|l1| Hh   |Vh, NULL, "IP_SRC_LOOKUP", O_IP_SRC_LOOKUP}
  , {      Ac|L3|l1| Hh   |Vh, NULL, "IP_DST_LOOKUP", O_IP_DST_LOOKUP} /* 3C */
#else
  , {      Ac|L2|l1| Hh   |Wh, NULL, "IP_SRC_LOOKUP", O_IP_SRC_LOOKUP}
  , {      Ac|L2|l1| Hh   |Wh, NULL, "IP_DST_LOOKUP", O_IP_DST_LOOKUP} /* 3C */
#endif
#if __FreeBSD_version >= 600000
  , {      Ac|L1|l1|        0, Wone, "ANTISPOOF"    , O_ANTISPOOF    } /* 3C */
  , {      Ac|L2|l2|       Wh, Wtre, "JAIL"         , O_JAIL         }
  , {   Nx|Aq|L2|l2|       Wh, Wone, "ALTQ"         , O_ALTQ         }
  , {      Ac|L1|l1| H2      , What, "DIVERTED"     , O_DIVERTED     } /* 40 */
  , {      Ac|Lm|l1|    Vv   , NULL, "TCPDATALEN"   , O_TCPDATALEN   } /* 40 */
  , {      Ac|L5|l5|       W6, Wone, "IP6_SRC"      , O_IP6_SRC      }
  , {      Ac|L1|l1|        0, Wone, "IP6_SRC_ME"   , O_IP6_SRC_ME   }
  , {      Ac|LI|l9|       V6, Wone, "IP6_SRC_MASK" , O_IP6_SRC_MASK } /* 44 */
  , {      Ac|L5|l5|       W6, Wone, "IP6_DST"      , O_IP6_DST      } /* 44 */
  , {      Ac|L1|l1|        0, Wone, "IP6_DST_ME"   , O_IP6_DST_ME   }
  , {      Ac|LI|l9|       V6, Wone, "IP6_DST_MASK" , O_IP6_DST_MASK }
  , {      Ac|Lm|l2| Hn   |Vh, NULL, "FLOW6ID"      , O_FLOW6ID      } /* 48 */
  , {      Ac|L8|l8|       Vb, Wone, "ICMP6TYPE"    , O_ICMP6TYPE    } /* 48 */
  , {      Ac|L1|l1| Hh      , NULL, "EXT_HDR"      , O_EXT_HDR      }
  , {      Ac|L1|l1|        0, Wone, "IP6"          , O_IP6          }
  , {   Nx|Aa|L1|l1| Hh      , NULL, "NETGRAPH"     , O_NETGRAPH     } /* 4C */
  , {   Nx|Aa|L1|l1| Hh      , NULL, "NGTEE"        , O_NGTEE        } /* 4C */
  , {      Ac|L1|l1|        0, Wone, "IP4"          , O_IP4          }
  , {   Nx|Aa|L1|l1| Hh      , NULL, "UNREACH6"     , O_UNREACH6     }
  , {   Nx|At|L1|l1| Hh      , NULL, "TAG"          , O_TAG          } /* 50 */
  , {      Ac|Lm|l1|    Vv   , NULL, "TAGGED"       , O_TAGGED       } /* 50 */
#endif
#if __FreeBSD_version >= 700100
  , {   Nx|Aa|L1|l1| Hr      , NULL, "SETFIB"       , O_SETFIB       }
  , {      Ac|L1|l1| Hr      , NULL, "FIB"          , O_FIB          }
#endif
  , {Er      |Lm|l1| Hh   |Vh, NULL, "#"            , O_LAST_OPCODE  }
} };

#if __FreeBSD_version >= 500000
__weak_reference(__bsdfig_s, bsdfig_s);
#endif
